{% extends "base.html" %}

{% load i18n discussions_tags %}

{% block content %}
    {% get_unread_message_count_for user as unread_message_count %}
    {% blocktrans %}{{ unread_message_count }} new messages.{% endblocktrans %}
    <ul>
        <li>
            <a href="{% url "discussions_list" %}">{% trans "Inbox" %}</a>
        </li>
        <li>
            <a href="{% url "discussions_sent" %}">{% trans "Messages sent" %}</a>
        </li>
        <li>
            <a href="{% url "discussions_deleted" %}">{% trans "Trash" %}</a>
        </li>
        <li>
            <a href="{% url "discussions_compose" %}">{% trans "Compose" %}</a>
        </li>
    </ul>
{% endblock %}
