{% extends "discussions/base.html" %}

{% load i18n %}

{% block content_title %}<h2>{% blocktrans with name=folder.name %}Folder "{{ name }}"{% endblocktrans %}</h2>{% endblock %}

{% block content %}
    {{ block.super }}

    <a href="{% url "discussions_folder_create" %}">{% trans "Create a new folder" %}</a>

    <ul>
        {% for recipient in recipient_list %}
            <li>
                {% include "discussions/_discussion.html" with discussion=recipient.discussion %}
            </li>
        {% endfor %}
    </ul>
{% endblock %}
