{% extends "discussions/base.html" %}
{% load i18n %}

{% block content_title %}<h2>{% trans "Messages" %}</h2>{% endblock %}

{% block content %}
    {{ block.super }}
    <form method="post" action="{% block form_action_url %}{% url "discussions_remove" %}{% endblock %}">
        {% csrf_token %}
        <ul id="discussion-list">
            {% for recipient in recipient_list %}
                <li>
                    {% include "discussions/_discussion.html" with discussion=recipient.discussion %}
                </li>
            {% endfor %}
        </ul>

        {% block actions %}
            {% if recipient_list %}
                <ul id="action-list">
                    <input type="submit" value="{% trans "Remove" %}" />
                </ul>
            {% endif %}
        {% endblock %}
    </form>
{% endblock %}
