{% extends "discussions/base.html" %}
{% load i18n %}

{% block content_title %}<h2>{% trans "Compose" %}</h2>{% endblock %}

{% block content %}
    {{ block.super }}
    <form action="" method="post" id="compose-form">
        {% csrf_token %}
        <fieldset>
            <legend>{% trans "Compose message" %}</legend>
            {{ form.as_p }}
        </fieldset>
        <input type="submit" name="submit" value="{% trans "Send" %}" />
    </form>
{% endblock %}
